<?php
class Xoogu_CurrencyConversion{
    
    //store the mysql table names
    protected $_tblCur, $_tblCou;
    //Options for this plugin
    protected $options = array();
    //Cache of currency info retrieved, stored as an associative array with the 3 letter ISO currency code used for the key
    protected $currencies = array();
    //As above, but using the currency symbol, country, and IP address as keys.
    protected $currenciesBySymbol = array(), $currenciesByCountry = array(), $currenciesByIP = array();
    //plugin version - remember to update plugin docblock header too
    public static $version = "1.1.0";
    
    /**
     *Initialise the plugin
     */
    public function __construct(){
		global $wpdb;
        $wpdb->show_errors();
        //mysql table names
        $this->_tblCur = $wpdb->base_prefix.'xoogu_currency_details';
        $this->_tblCou = $wpdb->base_prefix.'xoogu_currency_country';
        //get the options from the db
        $this->options = get_option('xoogu_currency_conversion');
        //register the shortcode handler
        add_shortcode('currencyConversion', array($this,'shortcodeHandler'));
        $this->loadAddOns();
    }
    
    
    /**
     *Loads add ons
     */
    public function loadAddOns(){
        
        if(empty($this->options['addOns'])){
            return;
        }
        foreach ($this->options['addOns'] as $addOn => $val){
            $addOn = dirname(__FILE__).'/add-ons/'.$addOn;
            if(file_exists($addOn)){
                include_once($addOn);
            }
        }
    }
    
    /**
     *Converts a given amount from one currency to another
     *@param string $from The currency code to convert from
     *@param string $to The currency code to convert to
     *@param int|float $amount (optional) The amount to be converted
     *@return int|float The converted amount
     */
    public function convertCurrency($from, $to, $amount=1){
        $base = $this->getCurrencyByCode($from);
        $quote = $this->getCurrencyByCode($to);
        if(empty($base->rate) || empty($quote->rate)){
            return false;
        }
        return $amount*($base->rate/$quote->rate);
    }
    
    
    /**
     *Converts a given amount from one currency to another
     *@param string $from The currency code to convert from
     *@param string $to The currency code to convert to
     *@param int|float $amount (optional) The amount to be converted
     *@param int $postID (optional) The id of the post that the static rates were saved for
     *
     *@return int|float The converted amount
     */
    public function convertCurrencyStatic($from, $to, $amount=1, $postID=null){
        if(empty($postID)){
            global $post;
            $postID=$post->ID;
        }
        $rates = get_post_meta($postID, '_xoogu_currency_conversion_rates', true);
        if(empty($rates[$from]) || empty($rates[$to])){
            return false;
        }
        return $amount*($rates[$from]/$rates[$to]);
    }
    
    
    /**
     *Gets information on a specific currency, such as exchange rate and symbol
     *@param string $code The 3 letter ISO currency code of the currency you want details for
     *@return object Object containing data on the currency
     */
    public function getCurrencyByCode($code){
        global $wpdb;
        
        //check cache
        if(!empty($this->currencies[$code])){
            return $this->currencies[$code];
        }
        
        //if cache is empty get from db
        if( ($this->currencies[$code] = $r = $wpdb->get_row($wpdb->prepare('SELECT 
			code, 	symbol, 	name, 	symbol_entity_num, 	symbol_entity_hex, 	full_name, 	dec_places, 	symbol_pref, 	rate,
			country, 	dec_point, 	thousands_sep, 	symbol_pos 
			FROM '.$this->_tblCur.' LEFT JOIN '.$this->_tblCou.' ON country = format_pref
			WHERE code = %s', $code))) ){
			
			//cache by country
			$this->currenciesByCountry[$r->country] = &$this->currencies[$code];
			
            //only cache by symbol if this is the preferred currency for that symbol
            if($r->symbol_pref){
                $this->currenciesBySymbol[$r->symbol] = &$this->currencies[$code];
            }
            return $r;
        }
    }
    
    
    /**
     *Gets currency details based on a provided currency symbol
     *Note that some currencies share symbols, the currency chosen will be the one that has symbol_pref set to 1 in the database
     *@param string $symbol The symbol e.g. $ of the currency you wish to lookup.
     *@return object The currency details
     */
    public function getCurrencyBySymbol($symbol){
        global $wpdb;
        
        //check cache
        if(!empty($this->currenciesBySymbol[$symbol])){
            return $this->currenciesBySymbol[$symbol];
        }
        
        //if cache empty get from db
        if( $r = $wpdb->get_row($wpdb->prepare('SELECT
			code, 	symbol, 	name, 	symbol_entity_num, 	symbol_entity_hex, 	full_name, 	dec_places, 	symbol_pref, 	rate,
			country, 	dec_point, 	thousands_sep, 	symbol_pos 
			FROM '.$this->_tblCur.' LEFT JOIN '.$this->_tblCou.' ON country = format_pref
			WHERE symbol = %s AND symbol_pref=1 LIMIT 1', $symbol)) ){
			
			//cache by code
			$this->currencies[$r->code] = $r;
			//cache by country
			$this->currenciesByCountry[$r->country] = &$this->currencies[$code];
			//cache by symbol
            return $this->currenciesBySymbol[$symbol] = &$this->currencies[$r->code];
        }
    }
    
    
    /**
     *Gets currency details based on the country specified
     *@param string $country The 2 letter ISO code for the country you want currency details for
     *@return object The currency details
     */
    public function getCurrencyByCountry($country){
        global $wpdb;
        
        //check the cache first
        if(!empty($this->currenciesByCountry[$country])){
            return $this->currenciesByCountry[$country];
        }
        
        //if cache empty get from db
        if( $r = $wpdb->get_row($wpdb->prepare('SELECT * FROM `'.$this->_tblCou.'`
			LEFT JOIN `'.$this->_tblCur.'` ON `currency` = `code`
			WHERE country = %s', $country)) ){
            
			//cache by code
			$this->currencies[$r->code] = $r;
			//only cache by symbol if this is the preferred currency for that symbol
            if($r->symbol_pref){
                $this->currenciesBySymbol[$r->symbol] = &$this->currencies[$code];
            }
			//cache by country
            return $this->currenciesByCountry[$country] = &$this->currencies[$r->code];
        }
    }
    
    
    /**
     *Gets currency details based on either a provided IP address or the user's IP address according to the $_SERVER['REMOTE_ADDR'] variable
     *@param string $ip (optional) The IP address you wish to lookup the currency for - uses $_SERVER['REMOTE_ADDR'] if not provided
     *@return object The currency details
     */
    public function getCurrencyByIP($ip=null){
        global $wpdb;
        
        //If no IP specified, get the user's IP 
        if(empty($ip)){
            $ip = $_SERVER['REMOTE_ADDR'];
        } 
        
        //check the cache first
        if(isset($this->currenciesByIP[$ip])){
            return $this->currenciesByIP[$ip];
        }
        
        //get the country code from the IP based on the method specified in the settings
        if(empty($this->options['geoFunc'])){
            return;
        }
        switch($this->options['geoFunc']){
            case 'server' :
                $country = $_SERVER['GEOIP_COUNTRY_CODE'];
                break;
            case 'php' :
                //PHP GeoIP extension - http://www.php.net/manual/en/function.geoip-country-code-by-name.php
                if(function_exists('geoip_country_code_by_name')){
                    $country = geoip_country_code_by_name($ip);
                }
                break;
            case 'geoip-detect' :
                //GeoIP Detection plugin - http://wordpress.org/plugins/geoip-detect/
                if(function_exists('geoip_detect_get_info_from_ip')){
                    if( ($userInfo = geoip_detect_get_info_from_ip($ip)) ){
                        $country = $userInfo->country_code;
                    }
                }
                break;
            case 'visitor-country' :
                //visitor-country - http://wordpress.org/plugins/visitor-country/
                if(isset($GLOBALS['VisitorCountry'])){
                    $country = $GLOBALS['VisitorCountry']->GetCode();
                }
                break;
            case 'ip-to-country' :
                //iptocountry plugin - http://wordpress.org/plugins/ip-to-country/
                if(class_exists('PepakIpToCountry')){
                    $country = PepakIpToCountry::IP_to_Country_XX($ip);
                }
                break;
            case 'statpress-visitors' :
                //statPress
                if(function_exists('geoip_open') && defined('GEOIP_STANDARD')){
                    $gi = geoip_open(luc_GeoIP_dbname('country'), GEOIP_STANDARD);
                    $country = geoip_country_code_by_addr($gi, $ip);
                }
                break;
            case 'ip2nation' :
                //Try ip2nation table - http://wordpress.org/plugins/wp-ip2nation-installer/
                if($wpdb->get_var("SHOW TABLES LIKE 'ip2nation'")) {
                    //ip2nation uses code of 'UK' for GB in main table. In ip2nationCountries the iso_code_2 is correct (GB), but Ukraine is given as UK (should be UA)
                    $sql = $wpdb->prepare('SELECT country FROM ip2nation WHERE ip < INET_ATON(%s) ORDER BY ip DESC LIMIT 0,1', $ip);
                    $country = $wpdb->get_var($sql);
                }
                break;
        }
        if(empty($country)){
            return;
        }
        
        //Lookup the currency from the country
        if($this->getCurrencyByCountry($country)){
            //create a reference to the details for caching by ip, and return the details
            return $this->currenciesByIP[$ip] = &$this->currenciesByCountry[$country];
        }
    }
    
    
    /**
     *Replaces a [currencyConversion] shortcode with the converted amount in the format
     *specified in the options
     */
    public function shortcodeHandler($attrs, $content){
        //get values from $attrs or defaults for a few vars
        extract(shortcode_atts( array('format' => $this->options['format'],
            'static' => isset($this->options['static']) ? $this->options['static'] : null,
            'dec_places' => isset($this->options['dec_places']) ? $this->options['dec_places'] : null
        ), $attrs));
        
        //if the static attribute was provided, we need to convert it from a string to a bool
        if(!empty($attrs['static'])){
            if(strcasecmp($attrs['static'], 'false') === 0){
                $static=false;
            }
            elseif(strcasecmp($attrs['static'], 'true') === 0){
                $static=true;
            }
        }
        
        //we remove any html entities from the content because the numbers in them might confuse the regex
        $contentDecoded = html_entity_decode ($content, ENT_QUOTES, 'UTF-8');
        
        //Get the currency we are converting from
        $from = false;
        //shortcode attributes
        if(!empty($attrs['from'])){
            $from = $attrs['from'];
        }
        //ISO currency code e.g. 10.50 USD
        elseif(preg_match('/[^a-zA-Z]([A-Z]{3,3})$/', $contentDecoded, $matches)){
            $from = $matches[1];
        }
        //ISO currency code e.g. USD10.50
        elseif(preg_match('/^([A-Z]{3,3})[^a-zA-Z]/', $contentDecoded, $matches)){
            $from = $matches[1];
        }
        //default option set?
        elseif(!empty($this->options['from'])){
            $from = $this->options['from'];
        }
        //currency sign e.g. $10.50
        elseif(preg_match('/^([^\s\d]{1,6})\d/', $contentDecoded, $matches)){
            $currencyDetails = $this->getCurrencyBySymbol($matches[1]);
            $from = $currencyDetails->code;
        }
        if(empty($from)){
            return $content;
        }
        //Get the currency we are converting to
        $to = false;
        //shortcode attributes
        if(!empty($attrs['to'])){
            $to = $this->getCurrencyByCode($attrs['to']);
        }
        //Currency can be manually set by adding a filter to 'xcc_currencyTo' and returning the code to convert to
        elseif($to = apply_filters('xcc_currencyTo', $to)){
            $to = $this->getCurrencyByCode($to);
        }
        //Otherwise get the currency by the user's IP address
        else{
            $to = $this->getCurrencyByIP();
        }
       
        //if we couldn't get the currency to convert to, return the original text
        //if the currency to convert from and to are the same, then return the original text
        if(empty($to) || $to->code == $from){
            return $content;
        }
        
        //First extract the number
        if( preg_match('/(\d([^\d]?\d)*)/', $contentDecoded, $matches) ){
            $a = $matches[0];
            //Check for separators
            if( preg_match_all('/[^\d]/', $a, $matches) ){
                $matches = $matches[0];
                //If we only have one separator
                if(count($matches) < 2){
                    //Is it a decimal point (i.e. has 2 digits after it)?
                    if( strlen($a) - (strrpos($a, $matches[0])+1) == 2 ){
                        //split the decimal from the number
                        $a = explode($matches[0], $a);
                        //convert back into a number
                        $a = $a[1] ? $a[0].'.'.$a[1] : $a[0];
                    }
                    else{
                        //strip thousand separators
                        $a = str_replace($matches[0], '', $a);
                    }
                }
                else{
                    //strip thousand separators
                    $a = str_replace($matches[0], '', $a);
                    //If we have two different seperators, last one must be decimal
                    if(end($matches) != $matches[0]){
                        //split the decimal from the number
                        $a = explode(end($matches), $a);
                        //convert back into a number
                        $a = $a[1] ? $a[0].'.'.$a[1] : $a[0];
                    }
                }
            }
        }
        
        //Get the converted amount
        if($static){ //if we are using static rates
            $a = $this->convertCurrencyStatic($from, $to->code, $a);
        }
        else{ //if we are using current rates
            $a = $this->convertCurrency($from, $to->code, $a);
        }
        //if we couldn't convert the amount, then don't do anything
        if($a === false){
            return $content;
        }
        
        //format the amount and return
        return $this->format($to, $a, $format, $content, $dec_places);
    }
    
    
    /**
     *Format an amount for a country based on the provided format string
     *@param stdClass $to Object containing the details for the country we are formatting the amount for
     *@param float $a Amount of money
     *@param string $format The amount will be formatted based on this string, see function details for list
     *@param string $content (optional) Any content to pass into the format, replaces %o in $format. Defaults to an empty string
     *@param int $dec_places (optional) The number of decimal places to format the amount to. Defaults to the standard number of decimal places for the currency provided.
     *@return string The amount formatted as per the provided $format string
     */
    public function format($to, $a, $format, $content='', $dec_places=null){
        if(!$this->isValidInteger($dec_places)){
            $dec_places=$to->dec_places;
        }
		 
		//use the local decimal point and thousands separator
		$a = number_format($a, (int)$dec_places, $to->dec_point, $to->thousands_sep);
		//reformat the $format string to place the currency symbol before or after tha amount as appropriate
		if($to->symbol_pos){
			$format = str_replace(array('%asy','%asn','%asx','%ac'), array('%a %sy','%a %sn','%a %sx','%a %c'), $format);
		}
		else{
			$format = str_replace(array('%asy','%asn','%asx','%ac'), array('%sy%a','%sn%a','%sx%a','%c %a'), $format);
		}
			
        //format the string
        /*
         %o = Original text
         %sy = symbol e.g. $
         %sn = symbol in html numerical entity format
         %sx = symbol in html hex entity format
         %n = name of currency e.g. dollars
         %fn = full name of currency e.g. United States Dollars
         %c = ISO currency code e.g. USD
         %a = The amount converted to local currency
         %asy = The amount converted to local currency with the symbol positioned before or after the amount as appropriate
         %asn = The amount converted to local currency with the symbol in html numerical entity format positioned before or after the amount as appropriate
         %asx = The amount converted to local currency with the symbol in html hex entity format positioned before or after the amount as appropriate
         %ac = The amount converted to local currency with the ISO currency code positioned before or after the amount as appropriate
        */
        return str_replace(array('%o', '%sy', '%sn', '%sx', '%n', '%fn', '%c', '%a'), array($content, $to->symbol, $to->symbol_entity_num, $to->symbol_entity_hex, $to->name, $to->full_name, $to->code, $a), $format);
    }
    
    
    /**
     *Purge all cached data in the object
     *Shouldn't be needed for anything.
     */
    public function purgeCache(){
        $this->currencies = $this->currenciesBySymbol = $this->currenciesByCountry = $this->currenciesByIP = array();
    }
	
    
	/**
	 *is_int returns false for string values (which is what we get through the attrs of a shortcode).
	 *To get round this, use this function to check if a value is an integer instead
	 *From http://www.webmasterworld.com/php/3189030.htm
	 *
	 *@param mixed $i value to check
	 *@return bool Whether the value is a valid integer
	 */
	public function isValidInteger($i){ 
        return (is_numeric($i) && (int)$i == $i);
	}
    
    
    /**
     *Gets a list of all the currencies stored in tblCur
     *@return array The list of currency codes
     */
    public function getCurrencyList(){
        //cache list
        static $currencyList;
        if(!empty($currencyList)){
            return $currencyList;
        }
        global $wpdb;
        return $currencyList=$wpdb->get_col('SELECT code FROM '.$this->_tblCur);
    }
    
    
    /**
     *Sets the options for this instance of the class
     *Note that the new options replace any existing options
     *@param array $options The options to store in the options property
     *@return Xoogu_CurrencyConversion this instance of the class (allows for chaining)
     */
    public function setOptions($options){
        foreach($options as $name => $val){
            $this->options[$name] = $val;
        }
        return $this;
    }
}